import wx

class wxConsole(wx.Panel):
    WELCOME_TEXT = \
"""
*******************************************************
* Welcome to PyLighter. This is the console.          *
* Anything printed by your program will show up here. *
* A text entry area is available below this box.      *
*******************************************************

"""

    def __init__(self, parent, id, stdin=None, stdout=None, style=0):
        wx.Panel.__init__(self,parent,id,style=style)
        self.stdin = stdin
        self.stdout = stdout
        
        f = wx.Font(10, wx.FONTFAMILY_TELETYPE, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        self.text_style = wx.TextAttr(font=f)
        
        self.output_area = wx.TextCtrl(self, -1, wxConsole.WELCOME_TEXT, style=wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH)
        self.input_area = wx.TextCtrl(self, -1, "", style=wx.TE_PROCESS_ENTER | wx.TE_RICH)
        self.go_button = wx.Button(self, -1, "Go")
        
        self.output_area.SetDefaultStyle(self.text_style)
        self.output_area.SetStyle(0,self.output_area.GetLastPosition(), self.text_style)
        self.input_area.SetDefaultStyle(self.text_style)
        
        self.output_area.SetInsertionPoint(self.output_area.GetLastPosition())
        
        self.out_box = wx.BoxSizer(wx.HORIZONTAL)
        self.out_box.Add(self.output_area, 1, wx.ALIGN_CENTER|wx.EXPAND)
        
        self.in_box = wx.BoxSizer(wx.HORIZONTAL)
        self.in_box.Add(self.input_area, 1, wx.ALIGN_TOP)
        self.in_box.Add((5,0))
        self.in_box.Add(self.go_button, 0, wx.ALIGN_TOP)
        
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.out_box, 1, wx.EXPAND|wx.ALL, 5)
        self.sizer.Add(self.in_box, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        
        self.Bind(wx.EVT_IDLE, self.OnIdle)
        self.Bind(wx.EVT_TEXT_ENTER, self.OnSendText, self.input_area)
        self.Bind(wx.EVT_BUTTON, self.OnSendText, self.go_button)
        
    def OnSendText(self,event):
        if self.stdin != None and not self.stdin.closed:
            try:
                text = "%s\n" % self.input_area.GetValue()
                self.output_area.AppendText(text)
                self.stdin.send(text)
                self.input_area.SetValue("")
            except (IOError, EOFError):
                self.stdin = None
            
            self.input_area.SetDefaultStyle(self.text_style) #For some reason this is getting reset when enter is pressed
    
    def set_stdin_pipe(self, stdin):
        if self.stdin != None and not self.stdin.closed:
            self.stdin.close()
        self.stdin = stdin
    
    def set_stdout_pipe(self, stdout):
        if self.stdout != None and not self.stdout.closed:
            self.stdout.close()
        self.stdout = stdout
    
    def OnIdle(self, event):
        if self.stdout != None and not self.stdout.closed:
            try:
                while self.stdout.poll(0.001):
                    self.output_area.AppendText(self.stdout.recv())
            except (IOError, EOFError):
                self.stdout = None

if __name__ == "__main__":
    import sys
    a = wx.App(0)
    f = wx.Frame(None,-1)
    c = wxConsole(f,-1,stdout=SharedFile)
    f.Show()
    a.MainLoop()
